package com.hb.controller;


import java.io.InputStream;
import java.net.URLEncoder;
import java.util.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import com.hb.util.DateUtil;
import com.hb.util.ServletUtils;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson2.JSON;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import com.hb.domain.AjaxResult;
import com.hb.util.MinioUtil;

@Api(tags = "Minio操作接口")
@RestController
public class MinioController {

    @Autowired
    MinioUtil minioUtil;

    /*@ApiOperation("列出所有的桶")
    @RequestMapping(value = "/listBuckets", method = RequestMethod.GET)
    @ResponseBody
    public AjaxResult listBuckets() throws Exception {
        return AjaxResult.success(minioUtil.listBuckets());
    }*/

    @ApiOperation("上传一个文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadfile", value = "MultipartFile文件对象", dataType="__file", paramType="form", required = true),
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(name = "objectName", value = "文件夹路径，例：20231215/002938", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @RequestMapping(value = "/uploadfile", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult fileupload(@RequestParam MultipartFile uploadfile, @RequestParam String bucket,
                                 @RequestParam(required = false) String objectName) throws Exception {
        minioUtil.createBucket(bucket);
        if (objectName != null) {
            minioUtil.uploadFile(uploadfile.getInputStream(), bucket, objectName + "/" + uploadfile.getOriginalFilename());
        } else {
            minioUtil.uploadFile(uploadfile.getInputStream(), bucket, uploadfile.getOriginalFilename());
        }
        return AjaxResult.success();
    }

    @ApiOperation("上传一个文件2")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadfile", value = "MultipartFile文件对象", dataType="__file", paramType="form", required = true),
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @RequestMapping(value = "/uploadfile2", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult fileupload2(@RequestParam MultipartFile uploadfile, @RequestParam String bucket) throws Exception {
        minioUtil.createBucket(bucket);
        String fileDir = buildDir();
        String fileDirName = buildDirName(fileDir, uploadfile.getOriginalFilename());
        minioUtil.uploadFile2(uploadfile, bucket, fileDirName);
        return AjaxResult.success();
    }

    @ApiOperation("递归列出一个桶中的所有文件和目录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @RequestMapping(value = "/listFiles", method = RequestMethod.GET)
    @ResponseBody
    public AjaxResult listFiles(@RequestParam String bucket) throws Exception {
        return AjaxResult.success("200", minioUtil.listFiles(bucket));
    }

    @ApiOperation("下载一个文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(name = "objectName", value = "文件全路径，例：20231215/002938/gh_03ace048af5d_344 (1) (2).jpg", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @RequestMapping(value = "/downloadFile", method = RequestMethod.GET)
    @ResponseBody
    public void downloadFile(@RequestParam String bucket, @RequestParam String objectName,
                             HttpServletResponse response) throws Exception {
        InputStream stream = minioUtil.download(bucket, objectName);
        ServletOutputStream output = response.getOutputStream();
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(objectName.substring(objectName.lastIndexOf("/") + 1), "UTF-8"));
        response.setContentType("application/octet-stream");
        response.setCharacterEncoding("UTF-8");
        IOUtils.copy(stream, output);
    }

    @ApiOperation("下载一个文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(name = "objectName", value = "文件全路径，例：20231215/002938/gh_03ace048af5d_344 (1) (2).jpg", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @RequestMapping(value = "/getPicture", method = RequestMethod.GET)
    @ResponseBody
    public void getLocalPicture(@RequestParam String bucket, @RequestParam String objectName,
                             HttpServletResponse response) {
        try {
            String contentType = minioUtil.getContenType(bucket, objectName);
            // 判断文件类型是否为图片
            boolean isImage = contentType != null && contentType.startsWith("image/");
            if (!isImage) {
                ServletUtils.renderString(response, JSON.toJSONString(AjaxResult.error("文件类型不是图片，无法查看！")));
            } else {
                InputStream stream = minioUtil.download(bucket, objectName);
                ServletOutputStream output = response.getOutputStream();
                //response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(objectName.substring(objectName.lastIndexOf("/") + 1), "UTF-8"));
                //response.setContentType("application/octet-stream");
                //response.setCharacterEncoding("UTF-8");
                IOUtils.copy(stream, output);
            }
        }catch (Exception e) {
            e.printStackTrace();
            ServletUtils.renderString(response, JSON.toJSONString(AjaxResult.error(e.getMessage())));
        }
    }


    @ApiOperation("删除一个文件")
    @RequestMapping(value = "/deleteFile", method = RequestMethod.GET)
    @ResponseBody
    public AjaxResult deleteFile(@RequestParam String bucket, @RequestParam String objectName) throws Exception {
        minioUtil.deleteObject(bucket, objectName);
        return AjaxResult.success();
    }

    @ApiOperation(value = "单个直传", notes = "单文件生成签名地址,前端直接传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileName", value = "文件名称", required = true),
            @ApiImplicitParam(name = "bucket", value = "桶名称", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @ApiResponses(
            {
                    @ApiResponse(code = 200, message = "生成地址成功"),
                    @ApiResponse(code = 1001, message = "生成失败,文件名不能为空"),
                    @ApiResponse(code = 1002, message = "生成失败,桶名不能为空"),
                    @ApiResponse(code = 1003, message = "生成地址出错"),
            }
    )
    @PostMapping("/buildOnePutURL")
    @ResponseBody
    public AjaxResult buildOnePutURL(String fileName, String bucket) {
        AjaxResult ajaxResult = new AjaxResult();
        if (StringUtils.isEmpty(fileName)) {
            ajaxResult.put("code", 1001);
            ajaxResult.put("msg", "生成失败,文件名不能为空");
            return ajaxResult;
        }
        if (StringUtils.isEmpty(bucket)) {
            ajaxResult.put("code", 1002);
            ajaxResult.put("msg", "生成失败,桶名不能为空");
            return ajaxResult;
        }

        try {
            String fileDir = buildDir();                   //目录
            String fileCode = UUID.randomUUID().toString(); //文件编号,放在数据库用
            //JSONObject jsonData = getOneURL(0, fileCode, fileDir, fileName, bucket);
            String fileDirName = buildDirName(fileDir, fileName);
            minioUtil.createBucket(bucket);
            String putURL = minioUtil.getOneURL(fileDirName, bucket);
            JSONObject jsonData = new JSONObject();
            jsonData.put("index", 0);
            jsonData.put("fileCode", fileCode);
            jsonData.put("fileDir", fileDir);
            jsonData.put("fileName", fileName);
            jsonData.put("fileDirName", fileDirName);
            jsonData.put("putURL", putURL);
            if (jsonData == null) {
                ajaxResult.put("code", 1003);
                ajaxResult.put("msg", "生成失败,Minio操作失败");
                return ajaxResult;
            }
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "success");
            ajaxResult.put("data", jsonData);
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.put("code", 500);
            ajaxResult.put("msg", "生成失败");
        }
        return ajaxResult;
    }

    /*@ApiOperation("删除一个桶")
    @RequestMapping(value = "/deleteBucket", method = RequestMethod.GET)
    @ResponseBody
    public AjaxResult deleteBucket(@RequestParam String bucket) throws Exception {
        minioUtil.deleteBucket(bucket);
        return AjaxResult.success();
    }*/

    @ApiOperation(value = "生成文件预览临时url", notes = "生成文件预览临时url（有效期7天）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileDirName", value = "文件全路径", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @ApiResponses(
            {
                    @ApiResponse(code = 200, message = "生成成功，返回文件预览url"),
                    @ApiResponse(code = 1001, message = "文件全路径不能为空"),
                    @ApiResponse(code = 1002, message = "桶名不能为空"),
                    @ApiResponse(code = 500, message = "生成失败"),
            }
    )
    @PostMapping("/getMinioURL")
    @ResponseBody
    public AjaxResult getMinioURL(String bucket, String fileDirName) {
        AjaxResult ajaxResult = new AjaxResult();
        if (StringUtils.isEmpty(fileDirName)) {
            ajaxResult.put("code", 1001);
            ajaxResult.put("msg", "生成失败,文件全路径不能为空");
            return ajaxResult;
        }
        if (StringUtils.isEmpty(bucket)) {
            ajaxResult.put("code", 1002);
            ajaxResult.put("msg", "生成失败,桶名不能为空");
            return ajaxResult;
        }
        try {
            String url = minioUtil.getMinioURL(bucket, fileDirName);
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "success");
            ajaxResult.put("url", url);
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.put("code", 500);
            ajaxResult.put("msg", "success");
        }
        return ajaxResult;
    }

    @ApiOperation(value = "生成多个文件预览临时url", notes = "生成多个文件预览临时url（有效期7天）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileDirNames", value = "文件全路径数组", required = true),
            @ApiImplicitParam(paramType = "header", name = "Authorization", value = "Bearer YOUR_TOKEN_HERE", required = true)
    })
    @ApiResponses(
            {
                    @ApiResponse(code = 200, message = "生成成功，返回文件预览url"),
                    @ApiResponse(code = 1001, message = "文件全路径不能为空"),
                    @ApiResponse(code = 1002, message = "文件全路径不能为空"),
                    @ApiResponse(code = 500, message = "生成失败"),
            }
    )
    @PostMapping("/getMinioURLs")
    public AjaxResult getMinioURLs(String bucket, String[] fileDirNames) {
        AjaxResult ajaxResult = new AjaxResult();
        if (fileDirNames == null || (fileDirNames != null && fileDirNames.length <= 0)) {
            ajaxResult.put("code", 1001);
            ajaxResult.put("msg", "文件全路径不能为空");
            return ajaxResult;
        }
        if (StringUtils.isEmpty(bucket)) {
            ajaxResult.put("code", 1002);
            ajaxResult.put("msg", "生成失败,桶名不能为空");
            return ajaxResult;
        }
        try {
            //List<String> urlList = minioUtil.getMinioURLs(bucket, fileDirNames);
            List<String> urlList = new ArrayList<>();
            for (String fileDirName : fileDirNames) {
                String url = minioUtil.getMinioURL(bucket, fileDirName);
                urlList.add(url);
            }
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "success");
            ajaxResult.put("urlList", urlList);
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.put("code", 500);
            ajaxResult.put("msg", "success");
        }
        return ajaxResult;
    }

    private String buildDir() {
        return DateUtil.getDateTime("yyyyMMdd/HHmmss");
    }

    public String buildDirName(String fileDir, String fileName) {
        return String.format("%s/%s", fileDir, fileName);
    }
}
