package com.hb.interceptor;

import com.hb.constant.Constants;
import com.hb.constant.HttpStatus;
import com.hb.domain.AjaxResult;
import com.hb.util.HttpUtils;
import com.hb.util.ServletUtils;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class MyInterceptor extends HandlerInterceptorAdapter {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        //在请求处理之前进行拦截处理
        //System.out.println("测试拦截器------preHandle------" + request.getRequestURL());
        //返回true表示继续执行请求，返回false表示停止执行请求;
        String token = request.getHeader("Authorization");
        if (StringUtils.hasText(token)){
            //验证token的合法性，根据自己的业务逻辑补充
            String result = HttpUtils.sendPost(Constants.VALIDATE_URL, "token=" + token);
            if (!StringUtils.isEmpty(result)) {
                JSONObject resultJSON = JSONObject.parseObject(result);
                int code = resultJSON.getIntValue("code");
                if (code == 200) {
                    return true;
                }
            }
        }
        //没有token，说明没有登录
        //使用response设置响应给前端的信息，根据自己的业务需求
        int code = HttpStatus.UNAUTHORIZED;
        String msg = com.hb.util.StringUtils.format("请求访问：{}，认证失败，无法访问系统资源", request.getRequestURI());
        ServletUtils.renderString(response, JSON.toJSONString(AjaxResult.error(msg)));
        return false;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        //在请求处理之后进行拦截处理
        //System.out.println("测试拦截器------postHandle");
    }
}
